/*
 * cooler.c
 *
 *  Created on: May 30, 2019
 *      Author: clayrosenthal and clay froelich
 */

#include "cooler.h"
#include "msp.h"
#include "UART.h"
#include "fan.h"
#include "temp.h"

#define TEMP_RANGE (160.0) // 10 << 4

int temp;
int fanSpeed;

void cooler_init()
{
    fan_init();
    UART_init();
    temp_init();

    temp = 0;
    fanSpeed = 0;

    // initialize terminal window
    SEND_UART_ESC("c");
    SEND_UART_ESC("c");
    SEND_UART_ESC("[H");
    sendUARTstr("Temp(C):\t");
    SEND_UART_NEWLINE;
    SEND_UART_NEWLINE;
    sendUARTstr("Current Threshold(C): 80");
    SEND_UART_NEWLINE;
    SEND_UART_NEWLINE;
    sendUARTstr("New Threshold(C): ");
    SEND_UART_NEWLINE;
    sendUARTstr("(Whole numbers only, 0-120 C)");
    SEND_UART_NEWLINE;
    SEND_UART_NEWLINE;
    sendUARTstr("Fan Speed(%):\t");
    SEND_UART_NEWLINE;
    sendUARTchar('|');
    SEND_UART_NEWLINE;
    sendUARTstr("|    |    |    |    |    |    |    |    |");
    SEND_UART_NEWLINE;
    sendUARTstr("0%       25%       50%       75%       100%"); // spaced by 10
    SEND_UART_NEWLINE;
    SEND_UART_NEWLINE;
    sendUARTstr("Clay^2 Cooler");
}

void makeBarGraph(uint16_t speed)
{
    SEND_UART_ESC("[9;2H");
    SEND_UART_ESC("[K"); // clears line
    for(; speed > 819; speed-=820)
        sendUARTchar('*');
}

void updateTemp(int newTemp)
{
    if(temp == newTemp)
        return;
    temp = newTemp;
    SEND_UART_ESC("[1;10H");
    SEND_UART_ESC("[K"); // clears line
    sendUARTint(temp >> 4);
    sendUARTchar('.');
    sendUARTchar(FORMULA_10TH(temp) + ZERO_CHR);
    sendUARTchar(FORMULA_100TH(temp) + ZERO_CHR);
    sendUARTchar(FORMULA_1000TH(temp) + ZERO_CHR);
    sendUARTchar(FORMULA_10000TH(temp) + ZERO_CHR);
}

void updateThreshold(int newThreshold)
{
    SEND_UART_ESC("[3;23H");
    SEND_UART_ESC("[K"); // clears line
    sendUARTint(newThreshold);
}

int calcDiff(uint16_t temp, uint16_t threshold)
{
    double diff;
    diff = (temp - threshold) / TEMP_RANGE;
    if(diff > 1)
    {
        return 100;
    }
    else if (diff < 0)
    {
        return 0;
    }
    return (int)(diff * 100);
}

void updateSpeed(int newSpeed)
{
    if(fanSpeed == newSpeed * 32765 / 100)
        return;
    fanSpeed = newSpeed * 32765 / 100;
    SEND_UART_ESC("[8;15H");
    SEND_UART_ESC("[K"); // clears line
    sendUARTint(newSpeed);
    sendUARTchar('%');
    set_speed(fanSpeed);
    makeBarGraph(fanSpeed);
}
