/*
 * temp.c
 *
 *  Created on: Jun 4, 2019
 *      Author: clay rosenthal and clay froelich
 */

#include "msp.h"
#include "temp.h"
#include "delay.h"
#include <stdint.h>
#include <stdio.h>
#include <string.h>

#define DATA_SIZE   (16)
uint8_t RXData[DATA_SIZE];
uint8_t flag;

void temp_init()
{

    // Configure GPIO
    P1->SEL0 |= SDA | SCL;                // I2C pins
    P1->SEL1 &= ~(SDA | SCL);
    P5->SEL0 &= ~ALERT;
    P5->SEL1 &= ~ALERT;
    P5->DIR &= ~ALERT;



    // Configure USCI_B0 for I2C mode
    EUSCI_B0->CTLW0 |= EUSCI_B_CTLW0_SWRST;       // Software reset enabled
    EUSCI_B0->CTLW0 = EUSCI_B_CTLW0_SWRST |       // Remain eUSCI in reset mode
                      EUSCI_B_CTLW0_MODE_3 |      // I2C mode
                      EUSCI_B_CTLW0_MST |         // Master mode
                      EUSCI_B_CTLW0_SYNC |        // Sync mode
                      EUSCI_B_CTLW0_SSEL__SMCLK;  // SMCLK

    EUSCI_B0->CTLW1 = EUSCI_B_CTLW1_ASTP_0;  // No Automatic stop generated

    set_DCO(FREQ_3MHZ);

    EUSCI_B0->BRW = 30;                      // baudrate = SMCLK / 30 = 100kHz
    EUSCI_B0->I2CSA = CONTROL;               // Slave address

    EUSCI_B0->CTLW0 &= ~EUSCI_B_CTLW0_SWRST;// Release eUSCI from reset

    EUSCI_B0->IE |= EUSCI_B_IE_RXIE      // Enable receive interrupt
                    | EUSCI_B_IE_TXIE         // Enable transmit interrupt
                    | EUSCI_B_IE_NACKIE // Enable NACK interrupt
                    ;
    // Enable global interrupt
    __enable_irq();

    // Enable eUSCIB0 interrupt in NVIC module
    NVIC->ISER[0] |= 1 << ((EUSCIB0_IRQn) & 31);

    flag = 0;

    WRITE_CONFIG(DEFAULT_CONFIG);

}

void sendAddress(uint8_t address)
{
    // Ensure stop condition got sent
    while (EUSCI_B0->CTLW0 & EUSCI_B_CTLW0_TXSTP);

    // I2C start condition
    EUSCI_B0->CTLW0 |= EUSCI_B_CTLW0_TR  // transmitter
                       | EUSCI_B_CTLW0_TXSTT; // start transmit

    while(!flag);
    flag = 0;
    EUSCI_B0->TXBUF = (address); // address
}


void sendData(uint8_t address, uint16_t data)
{
    sendAddress(address);


    while(!flag);
    flag = 0;
    EUSCI_B0->TXBUF = data >> 8;

    while(!flag);
    flag = 0;
    EUSCI_B0->TXBUF = data & BIT8_MASK;

    while(!flag);
    flag = 0;
    EUSCI_B0->CTLW0 |= EUSCI_B_CTLW0_TXSTP;

}

uint16_t readData(uint8_t address)
{
    uint16_t data;
    // Initialize data variables
    sendAddress(address);

    // wait for address to send
    while(!flag);
    flag = 0;

    // repeated start condition
    EUSCI_B0->CTLW0 &= ~EUSCI_B_CTLW0_TR; // Set receive mode (read)
    EUSCI_B0->CTLW0 |= EUSCI_B_CTLW0_TXSTT; // I2C start condition (restart)

    // Wait for start to be transmitted
    while ((EUSCI_B0->CTLW0 & EUSCI_B_CTLW0_TXSTT));


    while(!flag);
    flag = 0;
    data = EUSCI_B0->RXBUF;

    // send stop condition
    EUSCI_B0->CTLW0 |= EUSCI_B_CTLW0_TXSTP;

    while(!flag);
    flag = 0;
    data = (data << 8) | EUSCI_B0->RXBUF;

    return data;
}

// I2C interrupt service routine
void EUSCIB0_IRQHandler(void)
{
    if (EUSCI_B0->IFG & EUSCI_B_IFG_RXIFG0)
    {
        EUSCI_B0->IFG &= ~EUSCI_B_IFG_RXIFG0;

        flag = 1;
    }


    if (EUSCI_B0->IFG & EUSCI_B_IFG_TXIFG0)
    {
        EUSCI_B0->IFG &= ~EUSCI_B_IFG_TXIFG0;

        flag = 1;
    }

    if(EUSCI_B0->IFG & EUSCI_B_IFG_NACKIFG)
    {
        EUSCI_B0->IFG &= ~EUSCI_B_IFG_NACKIFG;

        EUSCI_B0->CTLW0 |= EUSCI_B_CTLW0_TXSTT; // I2C start condition (restart)
    }

}



